%% program computes the attitudes of an agent with preferences
%% exhibiting first-order risk aversion AND narrow framing
%% creates the bottom part of Figure 3

%% mean and standard deviation of the log return
%% on the non-financial asset
rm = 0.04;
rs = 0.03;
%% gross risk-free rate
rf = 1.02;
%% fraction of wealth theta_n_bar (th) invested
%% in the non-financial asset
th = 0.75;

%% parameters of the aggregator function W(.,.)
%% beta (bet) and rho (rho)
bet = 0.9;
rho = -1;

%% program looks at a range of values of the preference parameters
%% gamma (gm) and lambda (lam)
gm_min = 1.1;
gm_max = 10;
lam_min = 1.1;
lam_max = 4;
%% degree of narrow framing
b_0 = 0.1;

%% fineness of grid discretizing the (gamma,lambda) plane
lim = 40;

%% fineness of grid being used to discretize the area being integrated over
grd_fin = 1001;
%% the weighting vector for Simpson's rule
t1 = 4*kron(ones((grd_fin-3)/2,1),[1;0])+2*kron(ones((grd_fin-3)/2,1),[0;1]);
t1 = [1; t1; 4; 1];

%% arrays marking range of (gamma,lambda) plane for which various
%% gambles are accepted/rejected
gmb = zeros(lim,lim);
gmb1 = zeros(lim,lim);
gmb3 = zeros(lim,lim);

%% iterate through each (gamma,lambda) pair on the grid
for count = 1:lim;
   count
   for count2 = 1:lim;
      gm = gm_min + (count-1)*(gm_max-gm_min)/(lim-1);
      lam = lam_min + (count2-1)*(lam_max-lam_min)/(lim-1);
      
stp = 20/(grd_fin-1);
grd = [-10:stp:10]';
      
mu = (1-th)*rf+th*exp(rm+rs*grd);
mu = (mu.^(1-gm)).*exp(-grd.^2/2);
mu = (mu'*t1)*stp/(3*sqrt(2*pi));
      
%% compute consumption-wealth ratio in equation (48)
aph = 1 - bet^(1/(1-rho))*mu^(rho/((1-gm)*(1-rho)));
%% compute value function constant in equation (47)
A = (1-bet)^(1/rho)*aph^((rho-1)/rho);

%% consider first the case of IMMEDIATE gambles
%% gamble G_S, scaled

x = 550/1000000;
y = 500/1000000;

u1 = A*(((1+x)^(1-gm)/2+(1-y)^(1-gm)/2)^(1/(1-gm))-1)+b_0*(x-lam*y)/2;

%% gamble G_L, scaled

x = 20000000/100000;
y = 10000/100000;

u2 = A*(((1+x)^(1-gm)/2+(1-y)^(1-gm)/2)^(1/(1-gm))-1)+b_0*(x-lam*y)/2;

%% check where G_S is rejected and G_L is accepted
if u1 < 0;
   gmb(count,count2) = 1;
end;
if u2 > 0;
   gmb1(count,count2) = 1;
end;

%% now consider the case of DELAYED gambles

%% need to find the consumption-wealth ratio that maximizes
%% the expression in (50)
%% take the derivative of (50) and use interval bisection to find the root

%% now, DELAYED G_L

%% endpoints of interval that we search in
aph_0 = 0.01;
aph_1 = 0.99;

%% now evaluate the derivative of (50) at each endpoint
for ct = 1:2;

if ct == 1;
    aph2 = aph_0;
elseif ct == 2;
    aph2 = aph_1;
end;

%% gamble G_L, scaled
x2 = 20000000/((1-aph2)*100000);
y2 = 10000/((1-aph2)*100000);

%% compute the term in square brackets in (50)

stp = 20/(grd_fin-1);
grd = [-10:stp:10]';
    
w1 = (((1-th)*rf+th*exp(rm+rs*grd)+x2).^(1-gm)).*exp(-grd.^2/2);
w1 = (w1'*t1)*stp/(3*sqrt(2*pi));
w2 = (max((1-th)*rf+th*exp(rm+rs*grd)-y2,0.0000001).^(1-gm)).*exp(-grd.^2/2);
w2 = (w2'*t1)*stp/(3*sqrt(2*pi));
w3 = (((1-th)*rf+th*exp(rm+rs*grd)+x2).^(-gm)).*exp(-grd.^2/2);
w3 = (w3'*t1)*stp/(3*sqrt(2*pi));
w4 = (max((1-th)*rf+th*exp(rm+rs*grd)-y2,0.0000001).^(-gm)).*exp(-grd.^2/2);
w4 = (w4'*t1)*stp/(3*sqrt(2*pi));
w5 = (0.5*(w1+w2))^(1/(1-gm));
w6 = 0.5*w3*x2/(1-aph2) - 0.5*w4*y2/(1-aph2);
%% term in square brackets in (50)
w7 = A * w5 + b_0*(x2-lam*y2)/2;
%% now compute the derivative of (50)
w8 = A * w5^gm * w6 + b_0 * (x2-lam*y2)/(2*(1-aph2));
w9 = (1-bet)*aph2^(rho-1) - bet*(1-aph2)^(rho-1)*w7^rho + bet*(1-aph2)^rho*w7^(rho-1)*w8;

if ct == 1;
    f_0 = w9;
else;
    f_1 = w9;
end;

end;

%% if the derivative is still positive at the right-hand endpoint of 0.99
%% let the optimum be a consumption-wealth ratio of 1
%% and the value function constant A_hat (A2) be expression (50)
%% evaluated at that point
if f_1 > 0;
    A2 = (1-bet+bet*(b_0*0.5*(200-lam*0.1))^rho)^(1/rho);
else;
    %% otherwise use interval bisection to find the optimal
    %% consumption-wealth ratio
    flg = 1;
    while flg > 0.000001;
        aph_2 = (aph_0 + aph_1)/2;
        flg = abs(aph_2-aph_1);
        x2 = 20000000/((1-aph_2)*100000);
        y2 = 10000/((1-aph_2)*100000);
        stp = 20/(grd_fin-1);
        grd = [-10:stp:10]';
        w1 = (((1-th)*rf+th*exp(rm+rs*grd)+x2).^(1-gm)).*exp(-grd.^2/2);
        w1 = (w1'*t1)*stp/(3*sqrt(2*pi));
        w2 = (max((1-th)*rf+th*exp(rm+rs*grd)-y2,0.0000001).^(1-gm)).*exp(-grd.^2/2);
        w2 = (w2'*t1)*stp/(3*sqrt(2*pi));   
        w3 = (((1-th)*rf+th*exp(rm+rs*grd)+x2).^(-gm)).*exp(-grd.^2/2);
        w3 = (w3'*t1)*stp/(3*sqrt(2*pi));
        w4 = (max((1-th)*rf+th*exp(rm+rs*grd)-y2,0.0000001).^(-gm)).*exp(-grd.^2/2);
        w4 = (w4'*t1)*stp/(3*sqrt(2*pi));   
        w5 = (0.5*(w1+w2))^(1/(1-gm));
        w6 = 0.5*w3*x2/(1-aph_2) - 0.5*w4*y2/(1-aph_2);
        w7 = A * w5 + b_0*(x2-lam*y2)/2;
        w8 = A * w5^gm * w6 + b_0 * (x2-lam*y2)/(2*(1-aph_2));   
        w9 = (1-bet)*aph_2^(rho-1) - bet*(1-aph_2)^(rho-1)*w7^rho + bet*(1-aph_2)^rho*w7^(rho-1)*w8;
        f_2 = w9;
        if sign(f_1) == sign(f_2);
            aph_1 = aph_2;
            f_1 = f_2;
        else;
            aph_0 = aph_1;
            aph_1 = aph_2;
            f_0 = f_1;
            f_1 = f_2;
        end;
    end;    
    aph2 = aph_1;
    A2 = (1-bet)*aph2^rho + bet*(1-aph2)^rho*w7^rho;
    A2 = A2^(1/rho);
end;

%% check where G_S is rejected and G_L is accepted
if A2 > A;
   gmb3(count,count2) = 1;
end;

end;
end;

%% plots a graph
%% top panel shows where G_S is rejected
%% bottom panel shows were G_L is accepted

plt2 = [];

for count = 1:lim;
   for count2 = 1:lim;
      gm = gm_min+(count-1)*(gm_max-gm_min)/(lim-1);
      lam = lam_min+(count2-1)*(lam_max-lam_min)/(lim-1);
      if gmb3(count,count2) == 1;
         plt2 = [plt2; gm lam];
      end;
   end;
end;

figure;

subplot(2,1,2);
plot(plt2(:,1),plt2(:,2),'+');
axis([1 gm_max 1 lam_max]);
xlabel('\gamma');
ylabel('\lambda');
